@@ -1,4 +1,4 @@
-/* $Id: cleanup.c,v 1.5 2007/03/26 20:05:51 nicm Exp $ */
+/* $Id: cleanup.c,v 1.11 2007/07/25 21:52:45 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -34,25 +34,33 @@ TAILQ_HEAD(, cleanent)		 cleanlist;
 void
 cleanup_check(void)
 {
-	if (!TAILQ_EMPTY(&cleanlist))
-		fatalx("cleanup_check: list not empty");
+	struct cleanent	*cent;
+
+	if (!TAILQ_EMPTY(&cleanlist)) {
+		TAILQ_FOREACH(cent, &cleanlist, entry)
+		        log_debug("cleanup: %s", cent->path);
+		fatalx("list not empty");
+	}
 }
 
 void
 cleanup_purge(void)
 {
 	struct cleanent	*cent;
+	int		 saved_errno;
 
 	/*
 	 * This must be signal safe.
 	 */
 
+	saved_errno = errno;
  	TAILQ_FOREACH(cent, &cleanlist, entry) {
 		if (unlink(cent->path) != 0) {
 			write(STDERR_FILENO, "unlink failed\n", 14);
 			_exit(1);
 		}
 	}
+	errno = saved_errno;
 }
 
 void
@@ -63,7 +71,7 @@ cleanup_flush(void)
 
 	sigfillset(&set);
 	if (sigprocmask(SIG_BLOCK, &set, &oset) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 
 	while (!TAILQ_EMPTY(&cleanlist)) {
 		cent = TAILQ_FIRST(&cleanlist);
@@ -73,11 +81,11 @@ cleanup_flush(void)
 	}
 
 	if (sigprocmask(SIG_SETMASK, &oset, NULL) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 }
 
 void
-cleanup_register(char *path)
+cleanup_register(const char *path)
 {
 	sigset_t	 set, oset;
 	struct cleanent	*cent;
@@ -86,21 +94,24 @@ cleanup_register(char *path)
 	log_debug("cleanup_register: %s by %ld", path, (long) getpid());
 #endif
 
+	if (path == NULL || *path == '\0')
+		fatalx("empty path");
+
 	cent = xmalloc(sizeof *cent);
 	cent->path = xstrdup(path);
 
 	sigfillset(&set);
 	if (sigprocmask(SIG_BLOCK, &set, &oset) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 
 	TAILQ_INSERT_HEAD(&cleanlist, cent, entry);
 
 	if (sigprocmask(SIG_SETMASK, &oset, NULL) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 }
 
 void
-cleanup_deregister(char *path)
+cleanup_deregister(const char *path)
 {
 	sigset_t	 set, oset;
 	struct cleanent	*cent;
@@ -109,9 +120,12 @@ cleanup_deregister(char *path)
  	log_debug("cleanup_deregister: %s by %ld", path, (long) getpid());
 #endif
 
+	if (path == NULL || *path == '\0')
+		fatalx("empty path");
+
 	sigfillset(&set);
 	if (sigprocmask(SIG_BLOCK, &set, &oset) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 
  	TAILQ_FOREACH(cent, &cleanlist, entry) {
 		if (strcmp(cent->path, path) == 0) {
@@ -122,9 +136,9 @@ cleanup_deregister(char *path)
 		}
 	}
 
-	fatalx("cleanup_deregister: entry not found");
+	fatalx("entry not found");
 
 out:
 	if (sigprocmask(SIG_SETMASK, &oset, NULL) < 0)
-		fatal("sigprocmask");
+		fatal("sigprocmask failed");
 }
