@@ -1,4 +1,4 @@
-/* $Id: deliver-keep.c,v 1.2 2006/12/11 13:07:08 nicm Exp $ */
+/* $Id: deliver-keep.c,v 1.8 2007/03/28 19:59:57 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,23 +18,33 @@
 
 #include <sys/types.h>
 
+#include <string.h>
+
 #include "fdm.h"
+#include "deliver.h"
 
-int	 keep_deliver(struct deliver_ctx *, struct action *);
-char	*keep_desc(struct action *);
+int	 deliver_keep_deliver(struct deliver_ctx *, struct actitem *);
+void	 deliver_keep_desc(struct actitem *, char *, size_t);
 
-struct deliver deliver_keep = { DELIVER_INCHILD, keep_deliver, keep_desc };
+struct deliver deliver_keep = {
+	"keep",
+	DELIVER_INCHILD,
+	deliver_keep_deliver,
+	deliver_keep_desc
+};
 
 int
-keep_deliver(struct deliver_ctx *dctx, unused struct action *t)
+deliver_keep_deliver(struct deliver_ctx *dctx, unused struct actitem *ti)
 {
-	*dctx->decision = DECISION_KEEP;
+	struct mail	*m = dctx->mail;
+
+	m->decision = DECISION_KEEP;
 
 	return (DELIVER_SUCCESS);
 }
 
-char *
-keep_desc(unused struct action *t)
+void
+deliver_keep_desc(unused struct actitem *ti, char *buf, size_t len)
 {
-	return (xstrdup("keep"));
+	strlcpy(buf, "keep", len);
 }
