@@ -1,4 +1,4 @@
-/* $Id: fdm.c,v 1.132 2007/03/30 12:48:04 nicm Exp $ */
+/* $Id: fdm.c,v 1.169 2008/02/10 06:56:28 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,7 +17,6 @@
  */
 
 #include <sys/param.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/utsname.h>
@@ -30,25 +29,20 @@
 #include <limits.h>
 #include <paths.h>
 #include <pwd.h>
-#include <netdb.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 #include <unistd.h>
 
 #include "fdm.h"
 
-#ifdef DEBUG
-const char		*malloc_options = "AFGJPX";
+#if defined(__OpenBSD__) && defined(DEBUG)
+const char		*malloc_options = "AFGJPRX";
 #endif
 
-extern FILE		*yyin;
-extern int 		 yyparse(void);
-
 void			 sighandler(int);
-int			 load_conf(void);
-void			 usage(void);
 
 struct conf		 conf;
 
@@ -74,25 +68,10 @@ get_time(void)
 	struct timeval	 tv;
 
 	if (gettimeofday(&tv, NULL) != 0)
-		fatal("gettimeofday");
+		fatal("gettimeofday failed");
 	return (tv.tv_sec + tv.tv_usec / 1000000.0);
 }
 
-int
-load_conf(void)
-{
-        yyin = fopen(conf.conf_file, "r");
-        if (yyin == NULL)
-                return (1);
-
-        yyparse();
-
-        if (fclose(yyin) != 0)
-		return (1);
-
-        return (0);
-}
-
 void
 fill_info(const char *home)
 {
@@ -100,7 +79,7 @@ fill_info(const char *home)
 	uid_t		 uid;
 	char		 host[MAXHOSTNAMELEN];
 
-	uid = getuid();
+	uid = geteuid();
 	if (conf.info.valid && conf.info.last_uid == uid)
 		return;
 	conf.info.valid = 1;
@@ -121,10 +100,10 @@ fill_info(const char *home)
 
 	if (conf.info.host == NULL) {
 		if (gethostname(host, sizeof host) != 0)
-			fatal("gethostname");
+			fatal("gethostname failed");
 		conf.info.host = xstrdup(host);
 
-		fill_fqdn(host, &conf.info.fqdn, &conf.info.addr);
+		getaddrs(host, &conf.info.fqdn, &conf.info.addr);
 	}
 
 	if (home != NULL && *home != '\0')
@@ -150,52 +129,32 @@ fill_info(const char *home)
 }
 
 void
-fill_fqdn(char *host, char **fqdn, char **addr)
-{
-	char			 ni[NI_MAXHOST];
-	struct addrinfo		*ai;
-
-	*fqdn = *addr = NULL;
-
-	if (getaddrinfo(host, NULL, NULL, &ai) != 0)
-		return;
-
-	if (getnameinfo(ai->ai_addr,
-	    ai->ai_addrlen, ni, sizeof ni, NULL, 0, NI_NUMERICHOST) == 0)
-		xasprintf(addr, "[%s]", ni);
-
-	if (getnameinfo(ai->ai_addr,
-	    ai->ai_addrlen, ni, sizeof ni, NULL, 0, NI_NAMEREQD) == 0)
-		*fqdn = xstrdup(ni);
-}
-
-int
 dropto(uid_t uid)
 {
 	struct passwd	*pw;
 	gid_t		 gid;
 
-	if (uid == NOUSR || uid == 0)
-		return (0);
+	if (uid == (uid_t) -1 || uid == 0)
+		return;
 
 	pw = getpwuid(uid);
 	if (pw == NULL) {
-		endpwent();
 		errno = ESRCH;
-		return (1);
+		fatal("getpwuid failed");
 	}
 	gid = pw->pw_gid;
 	endpwent();
 
-	if (setgroups(1, &gid) != 0 ||
-	    setresgid(gid, gid, gid) != 0 || setresuid(uid, uid, uid) != 0)
-		return (1);
-
-	return (0);
+	if (setgroups(1, &gid) != 0)
+		fatal("setgroups failed");
+	if (setresgid(gid, gid, gid) != 0)
+		fatal("setresgid failed");
+	if (setresuid(uid, uid, uid) != 0)
+		fatal("setresuid failed");
 }
 
 int
-check_incl(char *name)
+check_incl(const char *name)
 {
 	u_int	i;
 
@@ -203,7 +162,7 @@ check_incl(char *name)
 		return (1);
 
 	for (i = 0; i < ARRAY_LENGTH(&conf.incl); i++) {
-		if (name_match(ARRAY_ITEM(&conf.incl, i, char *), name))
+		if (account_match(ARRAY_ITEM(&conf.incl, i), name))
 			return (1);
 	}
 
@@ -211,7 +170,7 @@ check_incl(char *name)
 }
 
 int
-check_excl(char *name)
+check_excl(const char *name)
 {
 	u_int	i;
 
@@ -219,7 +178,7 @@ check_excl(char *name)
 		return (0);
 
 	for (i = 0; i < ARRAY_LENGTH(&conf.excl); i++) {
-		if (name_match(ARRAY_ITEM(&conf.excl, i, char *), name))
+		if (account_match(ARRAY_ITEM(&conf.excl, i), name))
 			return (1);
 	}
 
@@ -257,8 +216,9 @@ use_account(struct account *a, char **cause)
 __dead void
 usage(void)
 {
-	printf("usage: %s [-klmnqv] [-a name] [-D name=value] [-f conffile]"
-	    " [-u user] [-x name] [fetch|poll]\n", __progname);
+	fprintf(stderr,
+	    "usage: %s [-klmnqv] [-a name] [-D name=value] [-f conffile] "
+	    "[-u user] [-x name] [fetch|poll|cache] [arguments]\n", __progname);
         exit(1);
 }
 
@@ -279,84 +239,52 @@ main(int argc, char **argv)
 	pid_t		 pid;
 	struct children	 children, dead_children;
 	struct child	*child;
-	struct io      **ios, *io;
+	struct io       *rio;
+	struct iolist	 iol;
 	double		 tim;
 	struct sigaction act;
 	struct msg	 msg;
 	struct msgbuf	 msgbuf;
 	size_t		 off;
-	struct macro	*macro;
+	struct strings	 macros;
 	struct child_fetch_data *cfd;
 #ifdef DEBUG
 	struct rule	*r;
 	struct action	*t;
+	struct cache	*cache;
 #endif
 
+	log_open(stderr, LOG_MAIL, 0);
+
 	memset(&conf, 0, sizeof conf);
 	TAILQ_INIT(&conf.accounts);
 	TAILQ_INIT(&conf.rules);
 	TAILQ_INIT(&conf.actions);
+	TAILQ_INIT(&conf.caches);
 	conf.max_size = DEFMAILSIZE;
 	conf.timeout = DEFTIMEOUT;
 	conf.lock_types = LOCK_FLOCK;
 	conf.impl_act = DECISION_NONE;
 	conf.purge_after = 0;
 	conf.file_umask = DEFUMASK;
-	conf.file_group = NOGRP;
+	conf.file_group = -1;
 	conf.queue_high = -1;
 	conf.queue_low = -1;
-
-	log_init(1);
+	conf.def_user = -1;
+	conf.cmd_user = -1;
+	conf.strip_chars = xstrdup(DEFSTRIPCHARS);
 
 	ARRAY_INIT(&conf.incl);
 	ARRAY_INIT(&conf.excl);
 
-        while ((opt = getopt(argc, argv, "a:D:f:klmnqu:vx:?")) != EOF) {
+	ARRAY_INIT(&macros);
+        while ((opt = getopt(argc, argv, "a:D:f:klmnqu:vx:")) != EOF) {
                 switch (opt) {
 		case 'a':
-			ARRAY_ADD(&conf.incl, xstrdup(optarg), char *);
+			ARRAY_ADD(&conf.incl, xstrdup(optarg));
 			break;
 		case 'D':
-			if (*optarg != '$' && *optarg != '%') {
-				log_warnx("invalid macro: %s", optarg);
-				exit(1);
-			}
-			ptr = strchr(optarg, '=');
-			if (ptr != NULL)
-				*ptr++ = '\0';
-			if (strlen(optarg) > MAXNAMESIZE) {
-				log_warnx("macro name too long: %s", optarg);
-				exit(1);
-			}
-
-			macro = xmalloc(sizeof *macro);
-			macro->fixed = 1;
-			strlcpy(macro->name, optarg, sizeof macro->name);
-			TAILQ_INSERT_HEAD(&macros, macro, entry);
-
-			if (*optarg == '$') {
-				macro->type = MACRO_STRING;
-				if (ptr == NULL)
-					macro->value.str = xstrdup("");
-				else
-					macro->value.str = xstrdup(ptr);
-				log_debug2("added -D macro \"%s\": %s",
-				    macro->name, macro->value.str);
-				break;
-			}
-
-			macro->type = MACRO_NUMBER;
-			if (ptr == NULL) {
-				macro->value.num = 0;
-				break;
-			}
-			macro->value.num = strtonum(ptr, 0, LLONG_MAX, &errstr);
-			if (errstr != NULL) {
-				log_warnx("number is %s: %s", errstr, ptr);
-				exit(1);
-			}
-			log_debug2("added -D macro \"%s\": %lld", macro->name,
-			    macro->value.num);
+			ARRAY_ADD(&macros, optarg);
 			break;
                 case 'f':
                         conf.conf_file = xstrdup(optarg);
@@ -377,41 +305,46 @@ main(int argc, char **argv)
 			user = optarg;
 			break;
                 case 'v':
-                        conf.debug++;
+			if (conf.debug != -1)
+				conf.debug++;
                         break;
 		case 'q':
-			conf.quiet = 1;
+			conf.debug = -1;
 			break;
 		case 'x':
-			ARRAY_ADD(&conf.excl, xstrdup(optarg), char *);
+			ARRAY_ADD(&conf.excl, xstrdup(optarg));
 			break;
-                case '?':
                 default:
                         usage();
                 }
         }
 	argc -= optind;
 	argv += optind;
-	if (conf.quiet)
-		conf.debug = 0;
 	if (conf.check_only) {
 		if (argc != 0)
 			usage();
 	} else {
-		if (argc != 1)
+		if (argc < 1)
 			usage();
-		if (strncmp(argv[0], "poll", strlen(argv[0])) == 0)
+		if (strncmp(argv[0], "poll", strlen(argv[0])) == 0) {
+			if (argc != 1)
+				usage();
 			op = FDMOP_POLL;
-		else if (strncmp(argv[0], "fetch", strlen(argv[0])) == 0)
+		} else if (strncmp(argv[0], "fetch", strlen(argv[0])) == 0) {
+			if (argc != 1)
+				usage();
 			op = FDMOP_FETCH;
+		} else if (strncmp(argv[0], "cache", strlen(argv[0])) == 0)
+			op = FDMOP_CACHE;
 		else
 			usage();
 	}
 
-	/* check the user */
+	/* Check the user. */
 	if (user != NULL) {
 		pw = getpwnam(user);
 		if (pw == NULL) {
+			endpwent();
 			n = strtonum(user, 0, UID_MAX, &errstr);
 			if (errstr != NULL) {
 				if (errno == ERANGE) {
@@ -429,26 +362,29 @@ main(int argc, char **argv)
 		endpwent();
 	}
 
-	/* start logging to syslog if necessary */
-	log_init(!conf.syslog);
+	/* Set debug level and start logging to syslog if necessary. */
+	if (conf.syslog)
+		log_open(NULL, LOG_MAIL, conf.debug);
+	else
+		log_open(stderr, LOG_MAIL, conf.debug);
 	tt = time(NULL);
 	log_debug("version is: %s " BUILD ", started at: %.24s", __progname,
 	    ctime(&tt));
 
-	/* and the OS version */
+	/* And the OS version. */
 	if (uname(&un) == 0) {
 		log_debug2("running on: %s %s %s %s", un.sysname, un.release,
 		    un.version, un.machine);
 	} else
 		log_debug2("uname: %s", strerror(errno));
 
-	/* save the home dir and misc user info */
+	/* Save the home dir and misc user info. */
 	fill_info(getenv("HOME"));
 	log_debug2("user is: %s, home is: %s", conf.info.user, conf.info.home);
 
-	/* find the config file */
+	/* Find the config file. */
 	if (conf.conf_file == NULL) {
-		/* if no file specified, try ~ then /etc */
+		/* If no file specified, try ~ then /etc. */
 		xasprintf(&conf.conf_file, "%s/%s", conf.info.home, CONFFILE);
 		if (access(conf.conf_file, R_OK) != 0) {
 			xfree(conf.conf_file);
@@ -462,13 +398,14 @@ main(int argc, char **argv)
 	}
 	if (geteuid() != 0 && (sb.st_mode & (S_IROTH|S_IWOTH)) != 0)
 		log_warnx("%s: world readable or writable", conf.conf_file);
-        if (load_conf() != 0) {
+        if (parse_conf(conf.conf_file, &macros) != 0) {
                 log_warn("%s", conf.conf_file);
 		exit(1);
 	}
+	ARRAY_FREE(&macros);
 	log_debug2("configuration loaded");
 
-	/* sort out queue limits */
+	/* Sort out queue limits. */
 	if (conf.queue_high == -1)
 		conf.queue_high = DEFMAILQUEUE;
 	if (conf.queue_low == -1) {
@@ -477,10 +414,21 @@ main(int argc, char **argv)
 			conf.queue_low = conf.queue_high - 1;
  	}
 
-	/* set the umask */
+	/* Set the umask. */
 	umask(conf.file_umask);
 
-	/* print proxy info */
+	/* Figure out default and command users. */
+	if (conf.def_user == (uid_t) -1) {
+		conf.def_user = geteuid();
+		if (conf.def_user == 0) {
+			log_warnx("no default user specified");
+			exit(1);
+		}
+	}
+	if (conf.cmd_user == (uid_t) -1)
+		conf.cmd_user = geteuid();
+
+	/* Print proxy info. */
 	if (conf.proxy != NULL) {
 		switch (conf.proxy->type) {
 		case PROXY_HTTP:
@@ -497,7 +445,7 @@ main(int argc, char **argv)
 		    conf.proxy->server.host, conf.proxy->server.port);
 	}
 
-	/* print some locking info */
+	/* Print some locking info. */
 	*tmp = '\0';
 	if (conf.lock_types == 0)
 		strlcpy(tmp, "none", sizeof tmp);
@@ -511,34 +459,34 @@ main(int argc, char **argv)
 	}
 	log_debug2("locking using: %s", tmp);
 
-	/* initialise and print headers and domains */
+	/* Initialise and print headers and domains. */
 	if (conf.headers == NULL) {
 		conf.headers = xmalloc(sizeof *conf.headers);
 		ARRAY_INIT(conf.headers);
-		ARRAY_ADD(conf.headers, xstrdup("to"), char *);
-		ARRAY_ADD(conf.headers, xstrdup("cc"), char *);
+		ARRAY_ADD(conf.headers, xstrdup("to"));
+		ARRAY_ADD(conf.headers, xstrdup("cc"));
 	}
-	strs = fmt_strings(NULL, conf.headers);
+	strs = fmt_strings("", conf.headers);
 	log_debug2("headers are: %s", strs);
 	xfree(strs);
 	if (conf.domains == NULL) {
-		conf.domains = xmalloc(sizeof *conf.headers);
+		conf.domains = xmalloc(sizeof *conf.domains);
 		ARRAY_INIT(conf.domains);
-		ARRAY_ADD(conf.domains, xstrdup(conf.info.host), char *);
+		ARRAY_ADD(conf.domains, xstrdup(conf.info.host));
 		if (conf.info.fqdn != NULL) {
 			ptr = xstrdup(conf.info.fqdn);
-			ARRAY_ADD(conf.domains, ptr, char *);
+			ARRAY_ADD(conf.domains, ptr);
 		}
 		if (conf.info.addr != NULL) {
-			ptr = xstrdup(conf.info.addr);
-			ARRAY_ADD(conf.domains, ptr, char *);
+			xasprintf(&ptr, "\\[%s\\]", conf.info.addr);
+			ARRAY_ADD(conf.domains, ptr);
 		}
 	}
-	strs = fmt_strings(NULL, conf.domains);
+	strs = fmt_strings("", conf.domains);
 	log_debug2("domains are: %s", strs);
 	xfree(strs);
 
-	/* print the other settings */
+	/* Print the other settings. */
 	*tmp = '\0';
 	off = 0;
 	if (conf.allow_many)
@@ -549,6 +497,8 @@ main(int argc, char **argv)
 		off = strlcat(tmp, "keep-all, ", sizeof tmp);
 	if (conf.del_big)
 		off = strlcat(tmp, "delete-oversized, ", sizeof tmp);
+	if (conf.verify_certs)
+		off = strlcat(tmp, "verify-certificates, ", sizeof tmp);
 	if (sizeof tmp > off && conf.purge_after > 0) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "purge-after=%u, ", conf.purge_after);
@@ -565,6 +515,10 @@ main(int argc, char **argv)
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "default-user=%lu, ", (u_long) conf.def_user);
 	}
+	if (sizeof tmp > off) {
+		off += xsnprintf(tmp + off, (sizeof tmp) - off,
+		    "command-user=%lu, ", (u_long) conf.cmd_user);
+	}
 	if (sizeof tmp > off && conf.impl_act != DECISION_NONE) {
 		if (conf.impl_act == DECISION_DROP)
 			s = "drop";
@@ -579,7 +533,7 @@ main(int argc, char **argv)
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "file-umask=%o%o%o, ", MODE(conf.file_umask));
 	}
-	if (sizeof tmp > off && conf.file_group != NOGRP) {
+	if (sizeof tmp > off && conf.file_group != (gid_t) -1) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "file-group=%lu, ", (u_long) conf.file_group);
 	}
@@ -592,12 +546,16 @@ main(int argc, char **argv)
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "lock-file=\"%s\", ", conf.lock_file);
 	}
+	if (sizeof tmp > off) {
+		off += xsnprintf(tmp + off, (sizeof tmp) - off,
+		    "strip-characters=\"%s\", ", conf.strip_chars);
+	}
 	if (off >= 2) {
 		tmp[off - 2] = '\0';
 		log_debug2("options are: %s", tmp);
 	}
 
-	/* save and print tmp dir */
+	/* Save and print tmp dir. */
 	s = getenv("TMPDIR");
 	if (s == NULL || *s == '\0')
 		s = _PATH_TMP;
@@ -615,7 +573,7 @@ main(int argc, char **argv)
 	}
 	log_debug2("using tmp directory: %s", conf.tmp_dir);
 
-	/* if -n, bail now, otherwise check there is something to work with */
+	/* If -n, bail now, otherwise check there is something to work with. */
 	if (conf.check_only)
 		exit(0);
         if (TAILQ_EMPTY(&conf.accounts)) {
@@ -627,6 +585,14 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	/* Change to handle cache ops. */
+	if (op == FDMOP_CACHE) {
+		argc--;
+		argv++;
+		cache_op(argc, argv);
+	}
+
+	/* Check for child user if root. */
 	if (geteuid() == 0) {
 		pw = getpwnam(CHILDUSER);
 		if (pw == NULL) {
@@ -636,14 +602,10 @@ main(int argc, char **argv)
 		conf.child_uid = pw->pw_uid;
 		conf.child_gid = pw->pw_gid;
 		endpwent();
-
-		if (conf.def_user == 0) {
-			log_warnx("no default user specified");
-			exit(1);
-		}
 	}
 
-	/* set up signal handlers */
+	/* Set up signal handlers. */
+	memset(&act, 0, sizeof act);
 	sigemptyset(&act.sa_mask);
 	sigaddset(&act.sa_mask, SIGINT);
 	sigaddset(&act.sa_mask, SIGTERM);
@@ -651,19 +613,19 @@ main(int argc, char **argv)
 
 	act.sa_handler = SIG_IGN;
 	if (sigaction(SIGPIPE, &act, NULL) < 0)
-		fatal("sigaction");
+		fatal("sigaction failed");
 	if (sigaction(SIGUSR1, &act, NULL) < 0)
-		fatal("sigaction");
+		fatal("sigaction failed");
 	if (sigaction(SIGUSR2, &act, NULL) < 0)
-		fatal("sigaction");
+		fatal("sigaction failed");
 
 	act.sa_handler = sighandler;
 	if (sigaction(SIGINT, &act, NULL) < 0)
-		fatal("sigaction");
+		fatal("sigaction failed");
 	if (sigaction(SIGTERM, &act, NULL) < 0)
-		fatal("sigaction");
+		fatal("sigaction failed");
 
-	/* check lock file */
+	/* Check lock file. */
 	lock = conf.lock_file;
 	if (lock == NULL) {
 		if (geteuid() == 0)
@@ -672,7 +634,7 @@ main(int argc, char **argv)
 			xasprintf(&lock, "%s/%s", conf.info.home, LOCKFILE);
 	}
 	if (*lock != '\0' && !conf.allow_many) {
-		lockfd = open(lock, O_WRONLY|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+		lockfd = xcreate(lock, O_WRONLY, -1, -1, S_IRUSR|S_IWUSR);
 		if (lockfd == -1 && errno == EEXIST) {
 			log_warnx("already running (%s exists)", lock);
 			exit(1);
@@ -691,7 +653,7 @@ main(int argc, char **argv)
 	COUNTFDS("parent");
 #endif
 
-	/* start the children and build the array */
+	/* Start the children and build the array. */
 	ARRAY_INIT(&children);
 	ARRAY_INIT(&dead_children);
 
@@ -724,55 +686,56 @@ main(int argc, char **argv)
 	tim = get_time();
 
 	res = 0;
-	ios = NULL;
+	ARRAY_INIT(&iol);
 	while (!ARRAY_EMPTY(&children)) {
 		if (sigint || sigterm)
 			break;
 
-		/* fill the io list */
-		ios = xrealloc(ios, ARRAY_LENGTH(&children), sizeof **ios);
+		/* Fill the io list. */
+		ARRAY_CLEAR(&iol);
 		for (i = 0; i < ARRAY_LENGTH(&children); i++) {
-			child = ARRAY_ITEM(&children, i, struct child *);
-			ios[i] = child->io;
+			child = ARRAY_ITEM(&children, i);
+			ARRAY_ADD(&iol, child->io);
 		}
 
-		/* poll the io list */
-		n = io_polln(ios, ARRAY_LENGTH(&children), &io, INFTIM, NULL);
+		/* Poll the io list. */
+		n = io_polln(
+		    ARRAY_DATA(&iol), ARRAY_LENGTH(&iol), &rio, INFTIM, NULL);
 		switch (n) {
 		case -1:
-			fatalx("parent: child socket error");
+			fatalx("child socket error");
 		case 0:
-			fatalx("parent: child socket closed");
+			fatalx("child socket closed");
 		}
 
 		while (!ARRAY_EMPTY(&children)) {
-			/* check all children for pending privsep messages */
+			/* Check all children for pending privsep messages. */
 			for (i = 0; i < ARRAY_LENGTH(&children); i++) {
-				child = ARRAY_ITEM(&children, i,struct child *);
+				child = ARRAY_ITEM(&children, i);
 				if (privsep_check(child->io))
 					break;
 			}
 			if (i == ARRAY_LENGTH(&children))
 				break;
 
-			/* and handle them if necessary */
+			/* And handle them if necessary. */
 			if (privsep_recv(child->io, &msg, &msgbuf) != 0)
-				fatalx("parent: privsep_recv error");
+				fatalx("privsep_recv error");
 			log_debug3("parent: got message type %d, id %u from "
 			    "child %ld", msg.type, msg.id, (long) child->pid);
 
 			if (child->msg(child, &msg, &msgbuf) == 0)
 				continue;
 
-			/* child has said it is ready to exit, tell it to */
+			/* Child has said it is ready to exit, tell it to. */
 			memset(&msg, 0, sizeof msg);
 			msg.type = MSG_EXIT;
 			if (privsep_send(child->io, &msg, NULL) != 0)
-				fatalx("parent: privsep_send error");
+				fatalx("privsep_send error");
 
-			/* wait for the child */
+			/* Wait for the child. */
 			if (waitpid(child->pid, &status, 0) == -1)
-				fatal("waitpid");
+				fatal("waitpid failed");
 			if (WIFSIGNALED(status)) {
 				res = 1;
 				log_debug2("parent: child %ld got signal %d",
@@ -792,15 +755,15 @@ main(int argc, char **argv)
 			io_free(child->io);
 			child->io = NULL;
 
-			ARRAY_REMOVE(&children, i, struct child *);
-			ARRAY_ADD(&dead_children, child, struct child *);
+			ARRAY_REMOVE(&children, i);
+			ARRAY_ADD(&dead_children, child);
 		}
 	}
-	xfree(ios);
+	ARRAY_FREE(&iol);
 
-	/* free the dead children */
+	/* Free the dead children. */
 	for (i = 0; i < ARRAY_LENGTH(&dead_children); i++) {
-		child = ARRAY_ITEM(&dead_children, i, struct child *);
+		child = ARRAY_ITEM(&dead_children, i);
 		if (child->data != NULL)
 			xfree(child->data);
 		xfree(child);
@@ -810,18 +773,18 @@ main(int argc, char **argv)
 	if (sigint || sigterm) {
 		act.sa_handler = SIG_IGN;
 		if (sigaction(SIGINT, &act, NULL) < 0)
-			fatal("sigaction");
+			fatal("sigaction failed");
 		if (sigaction(SIGTERM, &act, NULL) < 0)
-			fatal("sigaction");
+			fatal("sigaction failed");
 
 		if (sigint)
 			log_warnx("parent: caught SIGINT. stopping");
 		else if (sigterm)
 			log_warnx("parent: caught SIGTERM. stopping");
 
-		/* kill the children */
+		/* Kill the children. */
 		for (i = 0; i < ARRAY_LENGTH(&children); i++) {
-			child = ARRAY_ITEM(&children, i,struct child *);
+			child = ARRAY_ITEM(&children, i);
 			kill(child->pid, SIGTERM);
 
 			io_close(child->io);
@@ -830,12 +793,12 @@ main(int argc, char **argv)
 		}
 		ARRAY_FREE(&children);
 
-		/* and wait for them */
+		/* And wait for them. */
 		for (;;) {
 			if ((pid = wait(&status)) == -1) {
 				if (errno == ECHILD)
 					break;
-				fatal("wait");
+				fatal("wait failed");
 			}
 			log_debug2("parent: child %ld killed", (long) pid);
 		}
@@ -847,15 +810,29 @@ main(int argc, char **argv)
  	log_debug2("parent: finished, total time %.3f seconds", tim);
 
 out:
-	if (!conf.allow_many && *conf.lock_file != '\0') {
-		if (unlink(conf.lock_file) != 0)
-			fatal("unlink");
-	}
+	if (!conf.allow_many && *conf.lock_file != '\0')
+		unlink(conf.lock_file);
 
 #ifdef DEBUG
 	COUNTFDS("parent");
 
-	/* free everything */
+	/* Free everything. */
+	if (conf.proxy != NULL) {
+		if (conf.proxy->user != NULL)
+			xfree(conf.proxy->user);
+		if (conf.proxy->pass != NULL)
+ 			xfree(conf.proxy->pass);
+		if (conf.proxy->server.host != NULL)
+ 			xfree(conf.proxy->server.host);
+		if (conf.proxy->server.port != NULL)
+ 			xfree(conf.proxy->server.port);
+		xfree(conf.proxy);
+	}
+	while (!TAILQ_EMPTY(&conf.caches)) {
+		cache = TAILQ_FIRST(&conf.caches);
+		TAILQ_REMOVE(&conf.caches, cache, entry);
+		free_cache(cache);
+	}
 	while (!TAILQ_EMPTY(&conf.accounts)) {
 		a = TAILQ_FIRST(&conf.accounts);
 		TAILQ_REMOVE(&conf.accounts, a, entry);
@@ -882,6 +859,7 @@ out:
 	xfree(conf.conf_file);
 	xfree(conf.lock_file);
 	xfree(conf.tmp_dir);
+	xfree(conf.strip_chars);
 	free_strings(conf.domains);
 	ARRAY_FREEALL(conf.domains);
 	free_strings(conf.headers);
@@ -889,7 +867,7 @@ out:
 	free_strings(&conf.incl);
 	free_strings(&conf.excl);
 
-	xmalloc_report("parent");
+	xmalloc_report(getpid(), "parent");
 #endif
 
 	exit(res);
