@@ -1,4 +1,4 @@
-/* $Id: fdm.h,v 1.330 2007/09/25 21:01:52 nicm Exp $ */
+/* $Id: fdm.h,v 1.335 2007/10/25 09:02:54 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -189,7 +189,8 @@ TAILQ_HEAD(macros, macro);
 enum fdmop {
 	FDMOP_NONE = 0,
 	FDMOP_POLL,
-	FDMOP_FETCH
+	FDMOP_FETCH,
+	FDMOP_CACHE
 };
 
 /*
@@ -624,6 +625,7 @@ struct conf {
 	int			 allow_many;
 	int			 keep_all;
 	int			 no_received;
+	int			 no_create;
 	int			 verify_certs;
 	u_int			 purge_after;
 	enum decision		 impl_act;
@@ -639,6 +641,7 @@ struct conf {
 	int		         del_big;
 	u_int			 lock_types;
 	uid_t			 def_user;
+	uid_t			 cmd_user;
 
 	TAILQ_HEAD(, cache)	 caches;
 	TAILQ_HEAD(, account)	 accounts;
@@ -678,7 +681,6 @@ enum cmp {
 struct file {
 	FILE		*f;
 	int		 line;
-	int		 rule_line; /* XXX */
 	const char	*path;
 };
 ARRAY_DECL(files, struct file *);
@@ -725,7 +727,6 @@ extern struct file     *parse_file;
 extern struct strb     *parse_tags;
 int	 		parse_conf(const char *, struct strings *);
 __dead printflike1 void yyerror(const char *, ...);
-printflike1 void 	yywarn(const char *, ...);
 
 /* parse-fn.c */
 char		*expand_path(const char *);
@@ -763,6 +764,10 @@ int		 check_incl(const char *);
 int	         check_excl(const char *);
 int		 use_account(struct account *, char **);
 void		 fill_info(const char *);
+__dead void	 usage(void);
+
+/* cache-op.c */
+__dead void	 cache_op(int, char **);
 
 /* re.c */
 int		 re_compile(struct re *, const char *, int, char **);
@@ -827,14 +832,15 @@ struct io 	*connectproxy(struct server *, int, struct proxy *,
 struct io	*connectio(struct server *, int, const char *, int, char **);
 
 /* file.c */
-int printflike3	 mkpath(char *, size_t, const char *, ...);
-int		 vmkpath(char *, size_t, const char *, va_list);
+int printflike3	 ppath(char *, size_t, const char *, ...);
+int		 vppath(char *, size_t, const char *, va_list);
 int 		 openlock(const char *, int, u_int);
 int 		 createlock(const char *, int, uid_t, gid_t, mode_t, u_int);
 void 		 closelock(int, const char *, u_int);
 int		 locksleep(const char *, const char *, long long *);
 int 		 xcreate(const char *, int, uid_t, gid_t, mode_t);
 int		 xmkdir(const char *, uid_t, gid_t, mode_t);
+int		 xmkpath(const char *, uid_t, gid_t, mode_t);
 const char 	*checkmode(struct stat *, mode_t);
 const char 	*checkowner(struct stat *, uid_t);
 const char 	*checkgroup(struct stat *, gid_t);
@@ -874,9 +880,12 @@ int		 mail_deliver(struct mail_ctx *, struct msg *, struct msgbuf *);
 TDB_CONTEXT	*db_open(char *);
 void		 db_close(TDB_CONTEXT *);
 int		 db_add(TDB_CONTEXT *, char *);
+int		 db_remove(TDB_CONTEXT *, char *);
 int		 db_contains(TDB_CONTEXT *, char *);
 int		 db_size(TDB_CONTEXT *);
+int		 db_print(TDB_CONTEXT *, void (*)(const char *, ...));
 int		 db_expire(TDB_CONTEXT *, uint64_t);
+int		 db_clear(TDB_CONTEXT *);
 
 /* cleanup.c */
 void		 cleanup_check(void);
