@@ -1,4 +1,4 @@
-/* $Id: mail.c,v 1.96 2007/03/28 17:24:39 nicm Exp $ */
+/* $Id: mail.c,v 1.126 2007/09/19 09:05:40 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,11 +17,8 @@
  */
 
 #include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
 
 #include <ctype.h>
-#include <fcntl.h>
 #include <fnmatch.h>
 #include <limits.h>
 #include <pwd.h>
@@ -37,12 +34,12 @@ void	mail_free(struct mail *);
 int
 mail_open(struct mail *m, size_t size)
 {
-	m->size = size;
-	m->space = m->size;
-	m->body = -1;
+	m->size = 0;
+	m->space = IO_ROUND(size);
+	m->body = 0;
 
-	if ((m->base = shm_create(&m->shm, m->size)) == NULL)
-		return (1);
+	if ((m->base = shm_create(&m->shm, m->space)) == NULL)
+		return (-1);
  	SHM_REGISTER(&m->shm);
 
 	m->off = 0;
@@ -92,7 +89,7 @@ mail_receive(struct mail *m, struct msg *msg, int destroy)
 
 	memcpy(m, mm, sizeof *m);
 	if ((m->base = shm_reopen(&m->shm)) == NULL)
-		return (1);
+		return (-1);
  	SHM_REGISTER(&m->shm);
 
 	m->data = m->base + m->off;
@@ -140,165 +137,17 @@ int
 mail_resize(struct mail *m, size_t size)
 {
 	if (SIZE_MAX - m->off < size)
-		fatalx("resize_mail: SIZE_MAX - m->off < size");
+		fatalx("size too large");
 	while (m->space <= (m->off + size)) {
 		if ((m->base = shm_resize(&m->shm, 2, m->space)) == NULL)
-			return (1);
+			return (-1);
 		m->space *= 2;
 	}
 	m->data = m->base + m->off;
 	return (0);
 }
 
-char *
-rfc822_time(time_t t, char *buf, size_t len)
-{
-	struct tm	*tm;
-	size_t		 n;
-
-	tm = localtime(&t);
-	if ((n = strftime(buf, len, "%a, %d %b %Y %H:%M:%S %z", tm)) == 0)
-		return (NULL);
-	if (n == len)
-		return (NULL);
-	return (buf);
-}
-
-int printflike3
-printpath(char *buf, size_t len, const char *fmt, ...)
-{
-	va_list	ap;
-	int	n;
-
-	if (len > INT_MAX) {
-		errno = ENAMETOOLONG;
-		return (1);
-	}
-
-	va_start(ap, fmt);
-	n = xvsnprintf(buf, len, fmt, ap);
-	va_end(ap);
-
-	if ((size_t) n > len) {
-		errno = ENAMETOOLONG;
-		return (1);
-	}
-
-	return (0);
-}
-int
-openlock(const char *path, u_int locks, int flags, mode_t mode)
-{
-	char		*lock;
-	int	 	 fd, error;
-	struct flock	 fl;
-
-	if (locks & LOCK_DOTLOCK) {
-		xasprintf(&lock, "%s.lock", path);
- 		fd = open(lock, O_WRONLY|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
-		if (fd == -1) {
-			if (errno == EEXIST)
-				errno = EAGAIN;
-			xfree(lock);
-			return (-1);
-		}
-		close(fd);
-		cleanup_register(lock);
-	}
-
-	fd = open(path, flags, mode);
-
-	if (fd != -1 && locks & LOCK_FLOCK) {
-		if (flock(fd, LOCK_EX|LOCK_NB) != 0) {
-			if (errno == EWOULDBLOCK)
-				errno = EAGAIN;
-			goto error;
-		}
-	}
-
-	if (fd != -1 && locks & LOCK_FCNTL) {
-		memset(&fl, 0, sizeof fl);
-		fl.l_start = 0;
-		fl.l_len = 0;
-		fl.l_type = F_WRLCK;
-		fl.l_whence = SEEK_SET;
-		if (fcntl(fd, F_SETLK, &fl) == -1) {
-			/* fcntl already returns EAGAIN if needed */
-			goto error;
-		}
-	}
-
-	if (locks & LOCK_DOTLOCK)
-		xfree(lock);
-	return (fd);
-
-error:
-	error = errno;
-	close(fd);
-	if (locks & LOCK_DOTLOCK) {
-		if (unlink(lock) != 0)
-			fatal("unlink");
-		cleanup_deregister(lock);
-		xfree(lock);
-	}
-	errno = error;
-	return (-1);
-}
-
-void
-closelock(int fd, const char *path, u_int locks)
-{
-	char	*lock;
-
-	if (locks & LOCK_DOTLOCK) {
-		xasprintf(&lock, "%s.lock", path);
-		if (unlink(lock) != 0)
-			fatal("unlink");
-		cleanup_deregister(lock);
-		xfree(lock);
-	}
-
-	close(fd);
-}
-
-int
-checkperms(const char *hdr, const char *path, int *exists)
-{
-	struct stat	sb;
-	gid_t		gid;
-	mode_t		mode;
-
-	if (stat(path, &sb) != 0) {
-		if (errno == ENOENT) {
-			*exists = 0;
-			return (0);
-		}
-		return (1);
-	}
-	*exists = 1;
-
-	mode = (S_ISDIR(sb.st_mode) ? DIRMODE : FILEMODE) & ~conf.file_umask;
-	if ((sb.st_mode & DIRMODE) != mode) {
-		log_warnx("%s: %s: bad permissions: %o%o%o, should be %o%o%o",
-		    hdr, path, MODE(sb.st_mode), MODE(mode));
-	}
-
-	if (sb.st_uid != getuid()) {
-		log_warnx("%s: %s: bad owner: %lu, should be %lu", hdr, path,
-		    (u_long) sb.st_uid, (u_long) getuid());
-	}
-
-	gid = conf.file_group;
-	if (gid == NOGRP)
-		gid = getgid();
-	if (sb.st_gid != gid) {
-		log_warnx("%s: %s: bad group: %lu, should be %lu", hdr, path,
-		    (u_long) sb.st_gid, (u_long) gid);
-	}
-
-	return (0);
-}
-
+/* Initialise for iterating over lines. */
 void
 line_init(struct mail *m, char **line, size_t *len)
 {
@@ -313,6 +162,7 @@ line_init(struct mail *m, char **line, size_t *len)
 		*len = (ptr - *line) + 1;
 }
 
+/* Move to next line. */
 void
 line_next(struct mail *m, char **line, size_t *len)
 {
@@ -331,6 +181,7 @@ line_next(struct mail *m, char **line, size_t *len)
 		*len = (ptr - *line) + 1;
 }
 
+/* Remove specified header. */
 int
 remove_header(struct mail *m, const char *hdr)
 {
@@ -338,170 +189,309 @@ remove_header(struct mail *m, const char *hdr)
 	size_t	 len;
 
 	if ((ptr = find_header(m, hdr, &len, 0)) == NULL)
-		return (1);
+		return (-1);
 
-	/* include the \n */
-	len++;
-
-	/* remove the header */
+	/* Remove the header. */
 	memmove(ptr, ptr + len, m->size - len - (ptr - m->data));
 	m->size -= len;
- 	if (m->body != -1)
-		m->body -= len;
+	m->body -= len;
 
 	return (0);
 }
 
+/* Insert header, before specified header if not NULL, otherwise at end. */
 int printflike3
 insert_header(struct mail *m, const char *before, const char *fmt, ...)
 {
 	va_list		 ap;
 	char		*hdr, *ptr;
 	size_t		 hdrlen, len, off;
+	u_int		 newlines;
 
+	newlines = 1;
 	if (before != NULL) {
-		/* insert before header */
+		/* Insert before header. */
 		ptr = find_header(m, before, &len, 0);
 		if (ptr == NULL)
-			return (1);
+			return (-1);
 		off = ptr - m->data;
 	} else {
-		/* insert at the end */
-		if (m->body == -1)
-			off = m->size;
-		else if (m->body < 1)
+		/* Insert at the end. */
+		if (m->body == 0) {
+			/*
+			 * Creating the headers section. Insert at the start,
+			 * and add an extra newline.
+			 */
 			off = 0;
-		else
+			newlines++;
+		} else {
+			/*
+			 * Body points just after the blank line. Insert before
+			 * the blank line.
+			 */
 			off = m->body - 1;
+		}
 	}
 
+	/* Create the header. */
 	va_start(ap, fmt);
 	hdrlen = xvasprintf(&hdr, fmt, ap);
 	va_end(ap);
 
-	/* include the \n */
-	hdrlen++;
+	/* Include the newlines. */
+	hdrlen += newlines;
 
-	/* make space for the header */
+	/* Make space for the header. */
 	if (mail_resize(m, m->size + hdrlen) != 0) {
 		xfree(hdr);
-		return (1);
+		return (-1);
 	}
 	ptr = m->data + off;
 	memmove(ptr + hdrlen, ptr, m->size - off);
 
-	/* copy the header */
-	memcpy(ptr, hdr, hdrlen - 1);
-	ptr[hdrlen - 1] = '\n';
+	/* Copy the header. */
+	memcpy(ptr, hdr, hdrlen - newlines);
+	memset(ptr + hdrlen - newlines, '\n', newlines);
 	m->size += hdrlen;
- 	if (m->body != -1)
-		m->body += hdrlen;
+	m->body += hdrlen;
 
 	xfree(hdr);
 	return (0);
 }
 
+/*
+ * Find a header. If value is set, only the header value is returned, with EOL
+ * stripped
+ */
 char *
 find_header(struct mail *m, const char *hdr, size_t *len, int value)
 {
-	char	*ptr, *end, *out;
+	char	*ptr;
 	size_t	 hdrlen;
 
 	hdrlen = strlen(hdr) + 1; /* include : */
-
-	end = m->data + (m->body == -1 ? m->size : (size_t) m->body);
-	ptr = m->data;
-	if (hdrlen > (size_t) (end - ptr))
-		return (NULL);
-	while (ptr[hdrlen - 1] != ':' ||
-	    strncasecmp(ptr, hdr, hdrlen - 1) != 0) {
-		ptr = memchr(ptr, '\n', end - ptr);
-		if (ptr == NULL)
-			return (NULL);
-		ptr++;
-		if (hdrlen > (size_t) (end - ptr))
-			return (NULL);
-	}
-
-	out = ptr + hdrlen;
-	ptr = memchr(out, '\n', end - out);
-	if (ptr == NULL)
-		*len = end - out;
-	else
-		*len = ptr - out;
-
-	/* header must be followed by space */
-	if (!isspace((int) *out))
+	if (m->body < hdrlen || m->size < hdrlen)
 		return (NULL);
 
-	/* sort out what is actually returned */
-	if (value) {
-		/* strip any following space */
-		while (isspace((int) *out)) {
-			out++;
-			(*len)--;
+	line_init(m, &ptr, len);
+	while (ptr != NULL) {
+		if (ptr >= m->data + m->body)
+			return (NULL);
+		if (*len >= hdrlen && ptr[hdrlen - 1] == ':') {
+			if (strncasecmp(ptr, hdr, hdrlen - 1) == 0)
+				break;
 		}
-	} else {
-		/* move back to the start of the header */
-		out -= hdrlen;
-		(*len) += hdrlen;
+		line_next(m, &ptr, len);
 	}
+	if (ptr == NULL)
+		return (NULL);
+
+	/* If the entire header is wanted, return it. */
+	if (!value)
+		return (ptr);
+
+	/* Otherwise skip the header and following spaces. */
+	ptr += hdrlen;
+	*len -= hdrlen;
+	while (*len > 0 && isspace((u_char) *ptr)) {
+		ptr++;
+		(*len)--;
+	}
+
+	/* And trim newlines. */
+	while (*len > 0 && ptr[*len - 1] == '\n')
+		(*len)--;
 
 	if (len == 0)
 		return (NULL);
-
-	return (out);
+	return (ptr);
 }
 
+/* Match a header. Same as find_header but uses fnmatch. */
+char *
+match_header(struct mail *m, const char *patt, size_t *len, int value)
+{
+	char	*ptr, *last, *hdr;
+	size_t	 hdrlen;
+
+	line_init(m, &ptr, len);
+	while (ptr != NULL) {
+		if (ptr >= m->data + m->body)
+			return (NULL);
+
+		if ((last = memchr(ptr, ':', *len)) != NULL) {
+			hdrlen = last - ptr;
+			hdr = xmemstrdup(ptr, hdrlen);
+
+			if (fnmatch(patt, hdr, FNM_CASEFOLD) == 0)
+				break;
+
+			xfree(hdr);
+		}
+
+		line_next(m, &ptr, len);
+	}
+	if (ptr == NULL)
+		return (NULL);
+	xfree(hdr);
+
+	/* If the entire header is wanted, return it. */
+	if (!value)
+		return (ptr);
+
+	/* Include the : in the length. */
+	hdrlen++;
+
+	/* Otherwise skip the header and following spaces. */
+	ptr += hdrlen;
+	*len -= hdrlen;
+	while (*len > 0 && isspace((u_char) *ptr)) {
+		ptr++;
+		(*len)--;
+	}
+
+	/* And trim newlines. */
+	while (*len > 0 && ptr[*len - 1] == '\n')
+		(*len)--;
+
+	if (len == 0)
+		return (NULL);
+	return (ptr);
+}
+
+/*
+ * Find offset of body. The body is the offset of the first octet after the
+ * separator (\n\n), or zero.
+ */
+size_t
+find_body(struct mail *m)
+{
+	size_t	 len;
+	char	*ptr;
+
+	line_init(m, &ptr, &len);
+	while (ptr != NULL) {
+		if (len == 1 && *ptr == '\n') {
+			line_next(m, &ptr, &len);
+			/* If no next line, body is end of mail. */
+			if (ptr == NULL)
+				return (m->size);
+			/* Otherwise, body is start of line after separator. */
+			return (ptr - m->data);
+		}
+		line_next(m, &ptr, &len);
+	}
+	return (0);
+}
+
+/* Count mail lines. */
+void
+count_lines(struct mail *m, u_int *total, u_int *body)
+{
+	size_t	 len;
+	char	*ptr;
+	int	 flag;
+
+	flag = 0;
+	*total = *body = 0;
+
+	line_init(m, &ptr, &len);
+	while (ptr != NULL) {
+		if (flag)
+			(*body)++;
+		if (len == 1 && *ptr == '\n')
+			flag = 1;
+		(*total)++;
+		line_next(m, &ptr, &len);
+	}
+}
+
+/* Append line to mail. Used during fetching. */
+int
+append_line(struct mail *m, const char *line, size_t size)
+{
+	if (mail_resize(m, m->size + size + 1) != 0)
+		return (-1);
+	if (size > 0)
+		memcpy(m->data + m->size, line, size);
+	m->data[m->size + size] = '\n';
+	m->size += size + 1;
+	return (0);
+}
+
+/* Fill array of users from headers. */
 struct users *
 find_users(struct mail *m)
 {
 	struct passwd	*pw;
 	struct users	*users;
+	char		*ptr, *last, *data, *hdr, *dom, *aptr, *dptr, *line;
 	u_int	 	 i, j;
-	char		*hdr, *ptr, *dptr, *dom;
 	size_t	 	 len, alen;
 
 	users = xmalloc(sizeof *users);
 	ARRAY_INIT(users);
 
-	for (i = 0; i < ARRAY_LENGTH(conf.headers); i++) {
-		if (*ARRAY_ITEM(conf.headers, i, char *) == '\0')
-			continue;
+	line = NULL;
+	line_init(m, &ptr, &len);
+	while (ptr != NULL) {
+		if (ptr >= m->data + m->body)
+			break;
+		line = xmemstrdup(ptr, len);
 
-		hdr = find_header(m, ARRAY_ITEM(conf.headers, i, char *),
-		    &len, 1);
-		if (hdr == NULL || len == 0)
-			continue;
-		while (isspace((int) *hdr)) {
-			hdr++;
-			len--;
-		}
-		if (*hdr == '\0')
-			continue;
+		/* Find separator. */
+		data = strchr(line, ':');
+		if (data == NULL)
+			goto next;
+		*data++ = '\0';
+		while (isspace((u_char) *data))
+			data++;
+		while ((last = strrchr(data, '\n')) != NULL)
+			*last = '\0';
 
-		while (len > 0) {
-			ptr = find_address(hdr, len, &alen);
-			if (ptr == NULL)
+		/* Is this in the list of headers? */
+		for (i = 0; i < ARRAY_LENGTH(conf.headers); i++) {
+			hdr = ARRAY_ITEM(conf.headers, i);
+			if (*hdr == '\0')
+				continue;
+			if (fnmatch(hdr, line, FNM_CASEFOLD) == 0)
 				break;
+		}
+		if (i == ARRAY_LENGTH(conf.headers))
+			goto next;
+
+		/* Yes, try to find addresses. */
+		while (*data != '\0') {
+			aptr = find_address(data, strlen(data), &alen);
+			if (aptr == NULL)
+				break;
+			data = aptr + alen;
+
+			aptr = xmemstrdup(aptr, alen);
+			dptr = memchr(aptr, '@', alen);
+			*dptr++ = '\0';
 
-			dptr = ((char *) memchr(ptr, '@', alen)) + 1;
 			for (j = 0; j < ARRAY_LENGTH(conf.domains); j++) {
-				dom = ARRAY_ITEM(conf.domains, j, char *);
+				dom = ARRAY_ITEM(conf.domains, j);
 				if (fnmatch(dom, dptr, FNM_CASEFOLD) != 0)
 					continue;
-				*--dptr = '\0';
-				pw = getpwnam(ptr);
+
+				pw = getpwnam(aptr);
 				if (pw != NULL)
-					ARRAY_ADD(users, pw->pw_uid, uid_t);
+					ARRAY_ADD(users, pw->pw_uid);
 				endpwent();
-				*dptr++ = '@';
 				break;
 			}
 
-			len -= (ptr - hdr) + alen;
-			hdr = ptr + alen;
+			xfree(aptr);
 		}
+
+	next:
+		if (line != NULL)
+			xfree(line);
+		line = NULL;
+
+		line_next(m, &ptr, &len);
 	}
 
 	if (ARRAY_EMPTY(users)) {
@@ -509,56 +499,95 @@ find_users(struct mail *m)
 		xfree(users);
 		return (NULL);
 	}
-	return (weed_users(users));
+	return (users);
 }
 
 char *
-find_address(char *hdr, size_t len, size_t *alen)
+find_address(char *buf, size_t len, size_t *alen)
 {
-	char	*ptr;
-	size_t	 off, pos;
+	char	*ptr, *hdr, *first, *last;
 
-	for (off = 0; off < len; off++) {
-		switch (hdr[off]) {
-		case '"':
-			off++;
-			while (off < len && hdr[off] != '"')
-				off++;
-			if (off < len)
-				off++;
-			break;
-		case '<':
-			off++;
-			ptr = memchr(hdr + off, '>', len - off);
-			if (ptr == NULL)
+	/*
+	 * RFC2822 email addresses are stupidly complicated, so we just do a
+	 * naive match which is good enough for 99% of addresses used now. This
+	 * code is pretty inefficient.
+	 */
+
+	/* Duplicate the header as a string to work on it. */
+	if (len == 0)
+		return (NULL);
+	hdr = xmemstrdup(buf, len);
+
+	/* First, replace any sections in "s with spaces. */
+	ptr = hdr;
+	while (*ptr != '\0') {
+		if (*ptr == '"') {
+			while (*ptr != '"' && *ptr != '\0')
+				*ptr++ = ' ';
+			if (*ptr == '\0')
 				break;
-			*alen = ptr - (hdr + off);
-			for (pos = 0; pos < *alen; pos++) {
-				if (!isaddr(hdr[off + pos]))
-					break;
-			}
-			if (pos != *alen)
-				break;
-			ptr = hdr + off;
-			if (*alen == 0 || memchr(ptr, '@', *alen) == NULL)
-				break;
-			if (ptr[0] == '@' || ptr[*alen - 1] == '@')
-				break;
-			return (ptr);
 		}
+		ptr++;
 	}
 
-	/* no address found. try the whole header */
-	*alen = 0;
-	for (*alen = 0; *alen < len; (*alen)++) {
-		if (!isaddr(hdr[*alen]))
+	/*
+	 * Now, look for sections matching:
+	 * 	[< ][A-Za-z0-9._%+-]+@[A-Za-z0-9.\[\]-]+[> ,;].
+	 */
+#define isfirst(c) ((c) == '<' || (c) == ' ')
+#define islast(c) ((c) == '>' || (c) == ' ' || (c) == ',' || (c) == ';')
+#define isuser(c) (isalnum(c) || \
+	(c) == '.' || (c) == '_' || (c) == '%' || (c) == '+' || (c) == '-')
+#define isdomain(c) (isalnum(c) || \
+	(c) == '.' || (c) == '-' || (c) == '[' || (c) == ']')
+	ptr = hdr + 1;
+	for (;;) {
+		/* Find an @. */
+		if ((ptr = strchr(ptr, '@')) == NULL)
 			break;
+
+		/* Find the end. */
+		last = ptr + 1;
+		while (*last != '\0' && isdomain((u_char) *last))
+			last++;
+		if (*last != '\0' && !islast((u_char) *last)) {
+			ptr = last + 1;
+			continue;
+		}
+
+		/* Find the start. */
+		first = ptr - 1;
+		while (first != hdr && isuser((u_char) *first))
+			first--;
+		if (first != hdr && !isfirst((u_char) *first)) {
+			ptr = last + 1;
+			continue;
+		}
+
+		/* If the last is > the first must be < and vice versa. */
+		if (*last == '>' && *first != '<') {
+			ptr = last + 1;
+			continue;
+		}
+		if (*first == '<' && *last != '>') {
+			ptr = last + 1;
+			continue;
+		}
+
+		/* If not right at the start, strip first character. */
+		if (first != hdr)
+			first++;
+
+		/* Free header copy. */
+		xfree(hdr);
+
+		/* Have last and first, return the address. */
+		*alen = last - first;
+		return (buf + (first - hdr));
 	}
-	if (*alen == 0 || memchr(hdr + off, '@', *alen) == NULL)
-		return (NULL);
-	if (hdr[off] == '@' || hdr[*alen - 1] == '@')
-		return (NULL);
-	return (hdr);
+
+	xfree(hdr);
+	return (NULL);
 }
 
 void
@@ -567,21 +596,16 @@ trim_from(struct mail *m)
 	char	*ptr;
 	size_t	 len;
 
-	if (m->data == NULL || m->size < 5 || strncmp(m->data, "From ", 5) != 0)
+	if (m->data == NULL || m->body == 0 || m->size < 5)
+		return;
+	if (strncmp(m->data, "From ", 5) != 0)
 		return;
 
-	ptr = memchr(m->data, '\n', m->size);
-	if (ptr == NULL)
-		ptr = m->data + m->size;
-	else
-		ptr++;
-	len = ptr - m->data;
-
+	line_init(m, &ptr, &len);
 	m->size -= len;
 	m->off += len;
 	m->data = m->base + m->off;
-	if (m->body != -1)
-		m->body -= len;
+	m->body -= len;
 }
 
 char *
@@ -620,12 +644,12 @@ fill_wrapped(struct mail *m)
 	u_int		 n;
 
 	if (!ARRAY_EMPTY(&m->wrapped))
-		fatalx("fill_wrapped: mail already wrapped");
+		fatalx("already wrapped");
 
 	ARRAY_INIT(&m->wrapped);
 	m->wrapchar = '\0';
 
-	end = m->body == -1 ? m->size : (size_t) m->body;
+	end = m->body;
 	ptr = m->data;
 
 	n = 0;
@@ -638,12 +662,12 @@ fill_wrapped(struct mail *m)
 		if (off >= end)
 			break;
 
-		/* check if the line starts with whitespace */
-		if (!isblank((int) *ptr))
+		/* Check if the line starts with whitespace. */
+		if (!isblank((u_char) *ptr))
 			continue;
 
-		/* save the position */
-		ARRAY_ADD(&m->wrapped, off - 1, size_t);
+		/* Save the position. */
+		ARRAY_ADD(&m->wrapped, off - 1);
 		n++;
 	}
 
@@ -660,5 +684,5 @@ set_wrapped(struct mail *m, char ch)
 	m->wrapchar = ch;
 
 	for (i = 0; i < ARRAY_LENGTH(&m->wrapped); i++)
-		m->data[ARRAY_ITEM(&m->wrapped, i, size_t)] = ch;
+		m->data[ARRAY_ITEM(&m->wrapped, i)] = ch;
 }
