@@ -1,4 +1,4 @@
-/* $Id: match-string.c,v 1.24 2007/03/19 20:04:48 nicm Exp $ */
+/* $Id: match-string.c,v 1.29 2007/09/24 20:30:20 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -39,22 +39,19 @@ match_string_match(struct mail_ctx *mctx, struct expritem *ei)
 	struct match_string_data	*data = ei->data;
 	struct account			*a = mctx->account;
 	struct mail			*m = mctx->mail;
-	int				 res;
 	char				*s, *cause;
+	int				 n;
 
 	s = replacestr(&data->str, m->tags, m, &m->rml);
-	log_debug2("%s: matching \"%s\" to \"%s\"", a->name, s, data->re.str);
-
-	res = re_simple(&data->re, s, &cause);
-	xfree(s);
-
-	if (res == -1) {
+	if ((n = re_string(&data->re, s, NULL, &cause)) == -1) {
+		xfree(s);
 		log_warnx("%s: %s", a->name, cause);
 		xfree(cause);
 		return (MATCH_ERROR);
-	}
+	} 
+	xfree(s);
 
-	if (res == 0)
+	if (n == 0) 
 		return (MATCH_FALSE);
 	return (MATCH_TRUE);
 }
@@ -64,6 +61,6 @@ match_string_desc(struct expritem *ei, char *buf, size_t len)
 {
 	struct match_string_data	*data = ei->data;
 
-	xsnprintf(buf, len,
-	    "string \"%s\" to \"%s\"", data->str.str, data->re.str);
+	xsnprintf(
+	    buf, len, "string \"%s\" to \"%s\"", data->str.str, data->re.str);
 }
