@@ -1,4 +1,4 @@
-/* $Id: pop3-common.c,v 1.4 2007/09/25 19:07:18 nicm Exp $ */
+/* $Id: pop3-common.c,v 1.9 2007/12/12 08:05:33 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,7 @@
 
 #include <sys/types.h>
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -236,7 +237,7 @@ pop3_save(struct account *a)
 	if (data->path == NULL)
 		return (0);
 
-	if (mkpath(tmp, sizeof tmp, "%s.XXXXXXXXXX", data->path) != 0)
+	if (ppath(tmp, sizeof tmp, "%s.XXXXXXXXXX", data->path) != 0)
 		goto error;
 	if ((fd = mkstemp(tmp)) == -1)
 		goto error;
@@ -354,7 +355,7 @@ pop3_state_init(struct account *a, struct fetch_ctx *fctx)
 
 /* Connect state. */
 int
-pop3_state_connect(struct account *a, unused struct fetch_ctx *fctx)
+pop3_state_connect(struct account *a, struct fetch_ctx *fctx)
 {
 	struct fetch_pop3_data	*data = a->data;
 
@@ -451,6 +452,7 @@ pop3_state_first(struct account *a, struct fetch_ctx *fctx)
 {
 	struct fetch_pop3_data	*data = a->data;
 	char			*line;
+	u_int			 n;
 
 	if (pop3_getln(a, fctx, &line) != 0)
 		return (FETCH_ERROR);
@@ -459,7 +461,7 @@ pop3_state_first(struct account *a, struct fetch_ctx *fctx)
 	if (!pop3_okay(line))
 		return (pop3_bad(a, line));
 
-	if (sscanf(line, "+OK %u %*u", &data->num) != 1)
+	if (sscanf(line, "+OK %u %u", &data->num, &n) != 2)
 		return (pop3_invalid(a, line));
 	data->cur = 0;
 
@@ -511,7 +513,7 @@ pop3_state_cache2(struct account *a, struct fetch_ctx *fctx)
 {
 	struct fetch_pop3_data	*data = a->data;
 	struct fetch_pop3_mail	*aux;
-	char			*line;
+	char			*line, *ptr;
 	u_int			 n;
 
 	/* Parse response and add to server queue. */
@@ -527,6 +529,26 @@ pop3_state_cache2(struct account *a, struct fetch_ctx *fctx)
 			return (pop3_bad(a, line));
 		line = strchr(line, ' ') + 1;
 
+		/* 
+		 * Check UID validity. We are intolerant about validity since
+		 * accepting bad UIDs could potentially end up with UIDs that
+		 * conflict.
+		 */
+		if (*line == '\0') {
+			log_warnx("%s: empty UID", a->name);
+			return (FETCH_ERROR);
+		}
+		for (ptr = line; *ptr != '\0'; ptr++) {
+			if (*ptr < 0x21 || *ptr > 0x7e) {
+				log_warnx("%s: invalid UID: %s", a->name, line);
+				return (FETCH_ERROR);
+			}
+		}
+		if (ptr > line + 70) {
+				log_warnx("%s: UID too big: %s", a->name, line);
+				return (FETCH_ERROR);
+		}			
+
 		aux = xcalloc(1, sizeof *aux);
 		aux->idx = n;
 		aux->uid = xstrdup(line);
@@ -667,7 +689,7 @@ pop3_state_cache3(struct account *a, struct fetch_ctx *fctx)
 
 /* Next state. */
 int
-pop3_state_next(struct account *a, unused struct fetch_ctx *fctx)
+pop3_state_next(struct account *a, struct fetch_ctx *fctx)
 {
 	struct fetch_pop3_data	*data = a->data;
 	struct mail		*m = fctx->mail;
@@ -800,6 +822,7 @@ pop3_state_list(struct account *a, struct fetch_ctx *fctx)
 	struct mail		*m = fctx->mail;
 	struct fetch_pop3_mail	*aux = m->auxdata;
 	char			*line;
+	u_int			 n;
 
 	if (pop3_getln(a, fctx, &line) != 0)
 		return (FETCH_ERROR);
@@ -808,8 +831,10 @@ pop3_state_list(struct account *a, struct fetch_ctx *fctx)
 	if (!pop3_okay(line))
 		return (pop3_bad(a, line));
 
-	if (sscanf(line, "+OK %*u %zu", &data->size) != 1)
+	if (sscanf(line, "+OK %u %zu", &n, &data->size) != 2)
 		return (pop3_invalid(a, line));
+	if (n != aux->idx)
+		return (pop3_bad(a, line));
 
 	if (pop3_putln(a, "RETR %u", aux->idx) != 0)
 		return (FETCH_ERROR);
